@@ -1,5 +1,4 @@
 import json
-import logging
 from datetime import datetime
 from enum import Enum, IntEnum, auto
 
@@ -25,7 +24,6 @@ from .fields import LongURLField
 from .mixins import CeleryCallMethodsMixin
 from .signals import refresh_data_done
 
-logger = logging.getLogger(__name__)
 
 # Decimal fields must be returned as float
 DEC2FLOAT = psycopg2.extensions.new_type(
@@ -46,6 +44,7 @@ class FieldTypes(Enum):
     Float = auto()
     Boolean = auto()
     Undefined = auto()
+    Date = auto()
 
     def _generate_next_value_(self, name, start, count, last_values):
         return name.lower()
@@ -85,6 +84,7 @@ class GeometryTypes(IntEnum):
 
 class Source(PolymorphicModel, CeleryCallMethodsMixin):
     name = models.CharField(max_length=255, unique=True)
+    credit = models.TextField(blank=True)
     slug = models.SlugField(max_length=255, unique=True)
     description = models.TextField(blank=True)
 
@@ -92,6 +92,7 @@ class Source(PolymorphicModel, CeleryCallMethodsMixin):
     geom_type = models.IntegerField(choices=GeometryTypes.choices())
 
     settings = JSONField(default=dict)
+    report = JSONField(default=dict)
 
     task_id = models.CharField(null=True, max_length=255)
     task_date = models.DateTimeField(null=True)
@@ -124,29 +125,42 @@ class Source(PolymorphicModel, CeleryCallMethodsMixin):
         return super().save(*args, **kwargs)
 
     def refresh_data(self):
+        report = {}
         with transaction.atomic():
             layer = self.get_layer()
             begin_date = datetime.now()
             row_count = 0
+            total = 0
 
-            for row in self._get_records():
+            for i, row in enumerate(self._get_records()):
+                total += 1
                 geometry = row.pop(self.SOURCE_GEOM_ATTRIBUTE)
                 try:
                     identifier = row[self.id_field]
                 except KeyError:
-                    raise Exception(
-                        "Can't find identifier field in one or more records"
-                    )
+                    msg = "Can't find identifier field for this record"
+                    report["status"] = "Warning"
+                    report.setdefault("message", []).append(msg)
+                    report.setdefault("lines", {}).setdefault(f"{i}", []).append(msg)
+                    continue
                 self.update_feature(layer, identifier, geometry, row)
                 row_count += 1
-
             self.clear_features(layer, begin_date)
 
-        refresh_data_done.send_robust(
-            sender=self.__class__, layer=layer.pk,
-        )
+        self.report = report
+        if not row_count:
+            self.report["status"] = "Error"
+            self.save(update_fields=["report"])
+            raise Exception("Failed to refresh data")
 
-        return {"count": row_count}
+        refresh_data_done.send_robust(
+            sender=self.__class__,
+            layer=layer.pk,
+        )
+        if row_count == total:
+            self.report["status"] = "success"
+            self.save(update_fields=["report"])
+        return {"count": row_count, "total": total}
 
     @transaction.atomic
     def update_fields(self):
@@ -157,12 +171,14 @@ class Source(PolymorphicModel, CeleryCallMethodsMixin):
         for record in records:
             record.pop(self.SOURCE_GEOM_ATTRIBUTE)
 
-            for field_name, value in record.items():
+            for i, (field_name, value) in enumerate(record.items()):
                 is_new = False
 
                 if field_name not in fields:
                     field, is_new = self.fields.get_or_create(
-                        name=field_name, defaults={"label": field_name}
+                        name=field_name,
+                        defaults={"label": field_name},
+                        order=i,
                     )
                     field.sample = []
                     fields[field_name] = field
@@ -181,9 +197,12 @@ class Source(PolymorphicModel, CeleryCallMethodsMixin):
                         try:
                             value = value.decode()
                         except (UnicodeDecodeError, AttributeError):
-                            logger.warning(
-                                f"{field_name} couldn't be decoded for source {self.pk}"
-                            )
+                            msg = f"{field_name} couldn't be decoded for source {self.name}"
+                            self.report["status"] = "Warning"
+                            self.report.setdefault("lines", {}).setdefault(
+                                f"{i}", []
+                            ).append(msg)
+                            self.save()
                             continue
 
                     fields[field_name].sample.append(value)
@@ -231,17 +250,20 @@ class Field(models.Model):
     )
     level = models.IntegerField(default=0)
     sample = JSONField(default=list)
+    order = models.IntegerField(default=0)
 
     def __str__(self):
         return f"{self.name} ({self.source.name} - {self.data_type})"
 
     class Meta:
         unique_together = ["source", "name"]
+        ordering = ("order",)
 
 
 class PostGISSource(Source):
     db_host = models.CharField(
-        max_length=255, validators=[RegexValidator(regex=HOST_REGEX)],
+        max_length=255,
+        validators=[RegexValidator(regex=HOST_REGEX)],
     )
     db_port = models.IntegerField(default=5432)
     db_username = models.CharField(max_length=63)
@@ -260,13 +282,19 @@ class PostGISSource(Source):
 
     @property
     def _db_connection(self):
-        conn = psycopg2.connect(
-            user=self.db_username,
-            password=self.db_password,
-            host=self.db_host,
-            port=self.db_port,
-            dbname=self.db_name,
-        )
+        try:
+            conn = psycopg2.connect(
+                user=self.db_username,
+                password=self.db_password,
+                host=self.db_host,
+                port=self.db_port,
+                dbname=self.db_name,
+            )
+        except psycopg2.errors.OperationalError as err:
+            self.report["status"] = "Error"
+            self.report.setdefault("message", []).append(err.args[0])
+            self.save()
+            raise
         return conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
 
     def _get_records(self, limit=None):
@@ -290,7 +318,10 @@ class GeoJSONSource(Source):
         try:
             return json.load(self.file)
         except json.JSONDecodeError:
-            logger.info("Source's GeoJSON file is not valid")
+            msg = "Source's GeoJSON file is not valid"
+            self.report["status"] = "Error"
+            self.report.setdefault("message", []).append(msg)
+            self.save()
             raise
 
     def _get_records(self, limit=None):
@@ -299,7 +330,7 @@ class GeoJSONSource(Source):
         limit = limit if limit else len(geojson["features"])
 
         records = []
-        for record in geojson["features"][:limit]:
+        for i, record in enumerate(geojson["features"][:limit]):
             try:
                 records.append(
                     {
@@ -310,9 +341,11 @@ class GeoJSONSource(Source):
                     }
                 )
             except (ValueError, GDALException):
-                raise ValueError(
-                    f"One of source's record has bad geometry: {record['geometry']}"
-                )
+                msg = "The record geometry seems invalid."
+                self.report["status"] = "Warning"
+                self.report.setdefault("line", {}).setdefault(f"{i}", []).append(msg)
+                self.save()
+                raise ValueError(msg)
 
         return records
 
@@ -399,10 +432,13 @@ class CSVSource(Source):
                 encoding=self.settings["encoding"],
                 quotechar=quotechar,
             )
-        except pyexcel.exceptions.FileTypeNotSupported as err:
-            msg = "Source's CSV file is not valid"
-            logger.info(msg)
-            err.message = msg  # new message for the user
+        # Exception is raised if no parser found
+        except (pyexcel.exceptions.FileTypeNotSupported, Exception) as err:
+            msg = "Provided CSV file is invalid"
+            self.report["status"] = "Error"
+            self.report.setdefault("message", []).append(msg)
+            self.save()
+            err.args = (msg,)  # new message for the user
             raise
 
     def _get_records(self, limit=None):
@@ -418,7 +454,10 @@ class CSVSource(Source):
 
         records = []
         srid = self._get_srid()
-        for row in sheet:
+        row_count = 0
+        total = 0
+        for i, row in enumerate(sheet):
+            total += 1
             if self.settings["coordinates_field"] == "two_columns":
                 lat_field = self.settings["latitude_field"]
                 lng_field = self.settings["longitude_field"]
@@ -429,7 +468,12 @@ class CSVSource(Source):
                 ignored_field = (row.index(x), row.index(y), *ignored_columns)
             else:
                 lnglat_field = self.settings["latlong_field"]
-                x, y = self._extract_coordinates(row, sheet.colnames, [lnglat_field])
+                try:
+                    x, y = self._extract_coordinates(
+                        row, sheet.colnames, [lnglat_field]
+                    )
+                except ValueError:
+                    continue
                 coord_fields = (
                     (sheet.colnames.index(lnglat_field),)
                     if self.settings.get("use_header")
@@ -438,23 +482,49 @@ class CSVSource(Source):
                 ignored_field = (*coord_fields, *ignored_columns)
 
             cells = self._get_cells(sheet, row, ignored_field)
-            records.append(
-                {
-                    self.SOURCE_GEOM_ATTRIBUTE: GEOSGeometry(
-                        f"Point({x} {y})", srid=srid
-                    ),
-                    **cells,
-                }
+            try:
+                records.append(
+                    {
+                        self.SOURCE_GEOM_ATTRIBUTE: GEOSGeometry(
+                            f"Point({x} {y})", srid=srid
+                        ),
+                        **cells,
+                    }
+                )
+            except (ValueError, GDALException):
+                msg = f"One of source's record has invalid geometry: Point({x} {y}) srid={srid}"
+                self.report["status"] = "Warning"
+                self.report.setdefault("message", []).append(msg)
+                self.save()
+                # raise ValueError(msg)
+                continue
+            row_count += 1
+        if not row_count:
+            self.report["status"] = "Error"
+            self.report.setdefault("message", []).append(
+                "No record could be imported, check the report"
             )
+        elif row_count == total:
+            self.report["status"] = "Success"
         return records
 
     def _extract_coordinates(self, row, colnames, fields):
         coords = []
         for field in fields:
             # if no header, we expect index for the columns has been provided
-            field_index = (
-                colnames.index(field) if self.settings.get("use_header") else int(field)
-            )
+            try:
+                field_index = (
+                    colnames.index(field)
+                    if self.settings.get("use_header")
+                    else int(field)
+                )
+            except ValueError as err:
+                msg = f"{field} is not a valid coordinate field"
+                self.report["status"] = "Warning"
+                self.report.setdefault("message", []).append(msg)
+                self.save()
+                err.args = (msg,)
+                raise
             c = row[field_index]
             coords.append(c)
         if len(coords) == 2:
@@ -503,7 +573,16 @@ class CSVSource(Source):
         return self.SEPARATORS[name]
 
     def _get_srid(self):
-        return int(self.settings["coordinate_reference_system"].split("_")[1])
+        coordinate_reference_system = self.settings["coordinate_reference_system"]
+        try:
+            return int(coordinate_reference_system.split("_")[1])
+        except (IndexError, ValueError) as err:
+            msg = f"Invalid SRID: {coordinate_reference_system}"
+            self.report["status"] = "Error"
+            self.report.setdefault("message", []).append(msg)
+            self.save()
+            err.args = (msg,)
+            raise
 
     # properties are use by serializer for representation (reading operation)
     @property
